/*
 * Pnote.c
 *
 * Routine to print correct note and accidental given the
 * midi note value and track.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mtm.h"

//	in the table below, & and # indicate that the note should
//	appear with that accidental. +/- are ## and &&
//

static char *scales[30];

void loadNoteList(char *path) {
  FILE *in = fopen(path, "r");
  char buff[256], *p;
  int i = 0;

  if (in == NULL) {
    fprintf(stderr, "mtm: error - unable to load note list data from %s\n",
	    path);
    perror("fopen");
    exit(1);
  }
  while (fgets(buff, 256, in) != NULL) {
    p = buff;
    while (*p == ' ' || *p == '\t')
      p++;
    if (p[0] == '/' || p[0] == '\n' || p[0] == '\r')
      continue;			// ignore comments and blanks
    p[24] = 0;
    if (i >= 30) {
      fprintf(stderr, "mtm: error - too many note list lines in %s\n", path);
      exit(1);
    }
    scales[i++] = strdup(p);
  }
  if (i < 30) {
    fprintf(stderr, "mtm: error - not enough note list lines in %s\n", path);
    exit(1);
  }
  fclose(in);
}

extern FILE *out;

void setkeysig(Trackp t, int key) {
  int i;
  char *p;
  
  key += 7;
  if (key < 0 || key >= 30) {
    fprintf(stderr, "mtm: key outside range -7..+7\n");
    exit(1);
  }
  p = scales[key];
  for (i=0; i<128; i++) {
    if (p[0] == 0)
      p = scales[key];
    t->name[i] = p[0];
    t->acc[i] = p[1];
    t->accp[i] = 0;
    p += 2;
  }
}

void resetaccs(Trackp t) {
  int i;
  for (i=0; i<128; i++)
    t->accp[i] = ' ';
}

void getPitch(Trackp t, Eventp np) {
  int nval = np->pitch;
  int o = goctave(t, nval);
  int n = t->name[nval];
  int a = t->acc[nval];
  int z = 7 * o + (n - 'a');

  if (t->accp[z] == ' ') {
    if (a == '+' || a == '-')
      t->accp[z] = ' ';
    else
      t->accp[z] = a;
  }
  else if (t->accp[z] == a)
    a = ' ';
  else if (a == ' ') {
    a = 'n';
    t->accp[z] = a;
  }
  else if (a == '-') {
    a = '&';
    t->accp[z] = ' ';
  }
  else if (a == '+') {
    a = '#';
    t->accp[z] = ' ';
  }
  np->oct  = o;
  np->note = n;
  np->acc  = a;
}

void putNote(Trackp t, Eventp np) {
  fputc(np->note, out);
  if (np->acc == ' ' || (np->flags & F_TIEB) != 0)
    return;
  if (np->acc == '-' || np->acc == '+')
    return;
  fputc(np->acc, out);
}
